@@ -5,17 +5,17 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2004 Insecure.Com LLC. Nmap       *
- * is also a registered trademark of Insecure.Com LLC.  This program is    *
- * free software; you may redistribute and/or modify it under the          *
- * terms of the GNU General Public License as published by the Free        *
- * Software Foundation; Version 2.  This guarantees your right to use,     *
- * modify, and redistribute this software under certain conditions.  If    *
- * you wish to embed Nmap technology into proprietary software, we may be  *
- * willing to sell alternative licenses (contact sales@insecure.com).      *
- * Many security scanner vendors already license Nmap technology such as  *
- * our remote OS fingerprinting database and code, service/version         *
- * detection system, and port scanning code.                               *
+ * The Nmap Security Scanner is (C) 1996-2008 Insecure.Com LLC. Nmap is    *
+ * also a registered trademark of Insecure.Com LLC.  This program is free  *
+ * software; you may redistribute and/or modify it under the terms of the  *
+ * GNU General Public License as published by the Free Software            *
+ * Foundation; Version 2 with the clarifications and exceptions described  *
+ * below.  This guarantees your right to use, modify, and redistribute     *
+ * this software under certain conditions.  If you wish to embed Nmap      *
+ * technology into proprietary software, we sell alternative licenses      *
+ * (contact sales@insecure.com).  Dozens of software vendors already       *
+ * license Nmap technology such as host discovery, port scanning, OS       *
+ * detection, and version detection.                                       *
  *                                                                         *
  * Note that the GPL places important restrictions on "derived works", yet *
  * it does not provide a detailed definition of that term.  To avoid       *
@@ -24,9 +24,12 @@
  * following:                                                              *
  * o Integrates source code from Nmap                                      *
  * o Reads or includes Nmap copyrighted data files, such as                *
- *   nmap-os-fingerprints or nmap-service-probes.                          *
- * o Executes Nmap                                                         *
- * o Integrates/includes/aggregates Nmap into an executable installer      *
+ *   nmap-os-db or nmap-service-probes.                                    *
+ * o Executes Nmap and parses the results (as opposed to typical shell or  *
+ *   execution-menu apps, which simply display raw Nmap output and so are  *
+ *   not derivative works.)                                                * 
+ * o Integrates/includes/aggregates Nmap into a proprietary executable     *
+ *   installer, such as those produced by InstallShield.                   *
  * o Links to a library or executes a program that does any of the above   *
  *                                                                         *
  * The term "Nmap" should be taken to also include any portions or derived *
@@ -35,7 +38,7 @@
  * These restrictions only apply when you actually redistribute Nmap.  For *
  * example, nothing stops you from writing and selling a proprietary       *
  * front-end to Nmap.  Just distribute it by itself, and point people to   *
- * http://www.insecure.org/nmap/ to download Nmap.                         *
+ * http://nmap.org to download Nmap.                                       *
  *                                                                         *
  * We don't consider these to be added restrictions on top of the GPL, but *
  * just a clarification of how we interpret "derived works" as it applies  *
@@ -47,14 +50,23 @@
  * If you have any questions about the GPL licensing restrictions on using *
  * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
  * we also offer alternative license to integrate Nmap into proprietary    *
- * applications and appliances.  These contracts have been sold to many    *
- * security vendors, and generally include a perpetual license as well as  *
- * providing for priority support and updates as well as helping to fund   *
- * the continued development of Nmap technology.  Please email             *
+ * applications and appliances.  These contracts have been sold to dozens  *
+ * of software vendors, and generally include a perpetual license as well  *
+ * as providing for priority support and updates as well as helping to     *
+ * fund the continued development of Nmap technology.  Please email        *
  * sales@insecure.com for further information.                             *
  *                                                                         *
+ * As a special exception to the GPL terms, Insecure.Com LLC grants        *
+ * permission to link the code of this program with any version of the     *
+ * OpenSSL library which is distributed under a license identical to that  *
+ * listed in the included COPYING.OpenSSL file, and distribute linked      *
+ * combinations including the two. You must obey the GNU GPL in all        *
+ * respects for all of the code used other than OpenSSL.  If you modify    *
+ * this file, you may extend this exception to your version of the file,   *
+ * but you are not obligated to do so.                                     *
+ *                                                                         *
  * If you received these files with a written license agreement or         *
- * contract stating terms other than the (GPL) terms above, then that      *
+ * contract stating terms other than the terms above, then that            *
  * alternative license agreement takes precedence over these comments.     *
  *                                                                         *
  * Source is provided to this software because we believe users have a     *
@@ -65,7 +77,7 @@
  * Source code also allows you to port Nmap to new platforms, fix bugs,    *
  * and add new features.  You are highly encouraged to send your changes   *
  * to fyodor@insecure.org for possible incorporation into the main         *
- * distribution.  By sending these changes to Fyodor or one the            *
+ * distribution.  By sending these changes to Fyodor or one of the         *
  * Insecure.Org development mailing lists, it is assumed that you are      *
  * offering Fyodor and Insecure.Com LLC the unlimited, non-exclusive right *
  * to reuse, modify, and relicense the code.  Nmap will always be          *
@@ -79,19 +91,19 @@
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
- * General Public License for more details at                              *
- * http://www.gnu.org/copyleft/gpl.html .                                  *
+ * General Public License v2.0 for more details at                         *
+ * http://www.gnu.org/licenses/gpl-2.0.html , or in the COPYING file       *
+ * included with Nmap.                                                     *
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: main.cc,v 1.10 2004/03/12 01:59:04 fyodor Exp $ */
+/* $Id: main.cc 7640 2008-05-22 20:45:32Z fyodor $ */
+
+#include <signal.h>
 
 #include "nmap.h"
-#include "osscan.h"
-#include "scan_engine.h"
-#include "idle_scan.h"
-#include "timing.h"
 #include "NmapOps.h"
+#include "utils.h"
 
 #ifdef MTRACE
 #include "mcheck.h"
@@ -104,37 +116,32 @@
 struct Library *SocketBase = NULL, *MiamiBase = NULL, *MiamiBPFBase = NULL, *MiamiPCapBase = NULL;
 static const char ver[] = "$VER:" NMAP_NAME " v"NMAP_VERSION " [Amiga.sf]";
 
-BOOL OpenLibs(void) {
- if(!(    MiamiBase = OpenLibrary(MIAMINAME,21))) return FALSE;
- if(!(   SocketBase = OpenLibrary("bsdsocket.library", 4))) return FALSE;
- if(!( MiamiBPFBase = OpenLibrary(MIAMIBPFNAME,3))) return FALSE;
- if(!(MiamiPCapBase = OpenLibrary(MIAMIPCAPNAME,5))) return FALSE;
-return TRUE;
-}
-
-void CloseLibs(void) {
+static void CloseLibs(void) {
   if ( MiamiPCapBase ) CloseLibrary( MiamiPCapBase );
   if ( MiamiBPFBase  ) CloseLibrary(  MiamiBPFBase );
   if (  SocketBase   ) CloseLibrary(   SocketBase  );
   if (   MiamiBase   ) CloseLibrary(   MiamiBase   );
 }
+
+static BOOL OpenLibs(void) {
+ if(!(    MiamiBase = OpenLibrary(MIAMINAME,21))) return FALSE;
+ if(!(   SocketBase = OpenLibrary("bsdsocket.library", 4))) return FALSE;
+ if(!( MiamiBPFBase = OpenLibrary(MIAMIBPFNAME,3))) return FALSE;
+ if(!(MiamiPCapBase = OpenLibrary(MIAMIPCAPNAME,5))) return FALSE;
+ atexit(CloseLibs);
+ return TRUE;
+}
 #endif
 
 /* global options */
 extern NmapOps o;  /* option structure */
-extern char **environ;
 
-int main(int argc, char *argv[], char *envp[]) {
+int main(int argc, char *argv[]) {
   /* The "real" main is nmap_main().  This function hijacks control at the
      beginning to do the following:
-     1) Check if Nmap called under name listed in INTERACTIVE_NAMES or with
-     interactive.
+     1) Check if Nmap was called with --interactive.
      2) Start interactive mode or just call nmap_main
   */
-  char *interactive_names[] = INTERACTIVE_NAMES;
-  int numinames = sizeof(interactive_names) / sizeof(char *);
-  int nameidx;
-  char *nmapcalledas;
   char command[2048];
   int myargc, fakeargc;
   char **myargv = NULL, **fakeargv = NULL;
@@ -150,7 +157,7 @@ int main(int argc, char *argv[], char *envp[]) {
   char *endptr;
   int interactivemode = 0;
   int fd;
-  struct timeval tv;
+  int arglen = 0;
 
 #ifdef __amigaos__
 	if(!OpenLibs()) {
@@ -172,28 +179,25 @@ int main(int argc, char *argv[], char *envp[]) {
   mtrace();
 #endif
 
-  /* You never know when "random" numbers will come in handy ... */
-  gettimeofday(&tv, NULL);
-  srand((tv.tv_sec ^ tv.tv_usec) ^ getpid());
-
   /* Trap these sigs for cleanup */
 #if HAVE_SIGNAL
   signal(SIGINT, sigdie);
   signal(SIGTERM, sigdie);
+#ifndef WIN32
   signal(SIGHUP, sigdie); 
-
   signal(SIGCHLD, reaper);
 #endif
-
-  /* First we figure out whether the name nmap is called as qualifies it 
-     for interactive mode treatment */
-  nmapcalledas = strrchr(argv[0], '/');
-  if (!nmapcalledas) {
-    nmapcalledas = argv[0];
-  } else nmapcalledas++;
+#endif
 
   if ((cptr = getenv("NMAP_ARGS"))) {
-    snprintf(command, sizeof(command), "nmap %s", cptr);
+    if (Snprintf(command, sizeof(command), "nmap %s", cptr) >= (int) sizeof(command)) {
+        error("Warning: NMAP_ARGS variable is too long, truncated");
+    }
+    /* copy rest of command-line arguments */
+    for (i = 1; i < argc && strlen(command) + strlen(argv[i]) + 1 < sizeof(command); i++) {
+      strcat(command, " ");
+      strcat(command, argv[i]);
+    }
     myargc = arg_parse(command, &myargv);
     if (myargc < 1) {
       fatal("NMAP_ARG variable could not be parsed");
@@ -203,14 +207,6 @@ int main(int argc, char *argv[], char *envp[]) {
     return ret;
   }
 
-  for(nameidx = 0; nameidx < numinames; nameidx++) {
-    if (strcasecmp(nmapcalledas, interactive_names[nameidx]) == 0) {
-      printf("Entering Interactive Mode because argv[0] == %s\n", nmapcalledas);
-      interactivemode = 1;
-      break;
-    }
-  }
-
   if (interactivemode == 0 &&
       argc == 2 && strcmp("--interactive", argv[1]) == 0) {
     interactivemode = 1;
@@ -227,7 +223,7 @@ int main(int argc, char *argv[], char *envp[]) {
     }
     return nmap_main(argc, argv);
   }
-  /*  printf("\nStarting nmap V. %s by fyodor@insecure.org ( www.insecure.org/nmap/ )\n", VERSION);*/
+
   printf("\nStarting %s V. %s ( %s )\n", NMAP_NAME, NMAP_VERSION, NMAP_URL);
 
   printf("Welcome to Interactive Mode -- press h <enter> for help\n");
@@ -262,7 +258,8 @@ int main(int argc, char *argv[], char *envp[]) {
       nmap_main(myargc, myargv);
     } else if (*myargv[0] == '!') {
       cptr = strchr(command, '!');
-      system(cptr + 1);
+      int rc = system(cptr + 1);
+      if (rc < 1) printf("system() execution of command failed\n");
     } else if (*myargv[0] == 'd') {
       o.debugging++;
     } else if (strcasecmp(myargv[0], "f") == 0) {
@@ -283,19 +280,20 @@ int main(int argc, char *argv[], char *envp[]) {
 	      fatal("Bad arguments to f!");
 	    }	    
 	    strncpy(fakeargs, myargv[i], sizeof(fakeargs));
-	  } else if (strcasecmp(myargv[i], "--nmap_path") == 0) {
+	  } else if (optcmp(myargv[i], "--nmap-path") == 0) {
 	    if (++i > myargc -1) {
 	      fatal("Bad arguments to f!");
 	    }	    
 	    strncpy(nmappath, myargv[i], sizeof(nmappath));
 	  } else {
-	    if (strlen(nmapargs) + strlen(myargv[i]) + 1 < sizeof(nmapargs)) {
+	    arglen = strlen(nmapargs);
+	    if (arglen + strlen(myargv[i]) + 1 < sizeof(nmapargs)) {
 	      strcat(nmapargs, " ");
-	      strcat(nmapargs, myargv[i]);
+	      strncat(nmapargs, myargv[i], arglen - 1);
 	    } else fatal("Arguments too long.");
 	  }	 
 	}
-	/* First we stick our arguments into envp */
+
 	if (o.debugging) {
 	  error("Adding to environment: %s", nmapargs);
 	}
@@ -312,30 +310,37 @@ int main(int argc, char *argv[], char *envp[]) {
 	    if ((pptr = getenv("PATH"))) {
 	      Strncpy(path, pptr, sizeof(path));
 	      pptr = path;
+	      /* Get the name Nmap was called as. */
+	      char *nmapcalledas = path_get_basename(argv[0]);
+	      if (nmapcalledas == NULL)
+		pfatal("Could not get nmap executable basename");
 	      while(pptr && *pptr) {
 		endptr = strchr(pptr, ':');
 		if (endptr) { 
 		  *endptr = '\0';
 		}
-		snprintf(nmappath, sizeof(nmappath), "%s/%s", pptr, nmapcalledas);
+		Snprintf(nmappath, sizeof(nmappath), "%s/%s", pptr, nmapcalledas);
 		if (stat(nmappath, &st) != -1)
 		  break;
 		nmappath[0] = '\0';
 		if (endptr) pptr = endptr + 1;
 		else pptr = NULL;
 	      }
+	      free(nmapcalledas);
 	    }
 	  }
 	}
 	if (!*nmappath) {
-	  fatal("Could not find Nmap -- you must add --nmap_path argument");
+	  fatal("Could not find Nmap -- you must add --nmap-path argument");
 	}       
 
 	/* We should be courteous and give Nmap reasonable signal defaults */
 #if HAVE_SIGNAL
 	signal(SIGINT, SIG_DFL);
 	signal(SIGTERM, SIG_DFL);
+#ifndef WIN32
 	signal(SIGHUP, SIG_DFL);
+#endif
 	signal(SIGSEGV, SIG_DFL);
 #endif
 
@@ -347,7 +352,7 @@ int main(int argc, char *argv[], char *envp[]) {
 	  }
 	} else {
 	  fakeargc = 1;
-	  fakeargv = (char **) malloc(sizeof(char *) * 2);
+	  fakeargv = (char **) safe_malloc(sizeof(char *) * 2);
 	  fakeargv[0] = nmappath;
 	  fakeargv[1] = NULL;
 	}
@@ -363,7 +368,7 @@ int main(int argc, char *argv[], char *envp[]) {
 	}
 
 	/* OK, I think we are finally ready for the big exec() */
-	ret = execve(nmappath, fakeargv, environ);
+	ret = execv(nmappath, fakeargv);
 	if (ret == -1) {
 	  pfatal("Could not exec %s", nmappath);
 	}
@@ -381,9 +386,6 @@ int main(int argc, char *argv[], char *envp[]) {
     }
     arg_parse_free(myargv);
   }
-#ifdef __amigaos__
-  CloseLibs();
-#endif
   return 0;
 
 }
