@@ -54,8 +54,9 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: filespace.c 7327 2008-05-05 04:10:20Z fyodor $ */
+/* $Id: filespace.c 8430 2008-06-22 04:56:02Z bmenrigh $ */
 
+#include "nsock_internal.h"
 #include "filespace.h"
 
 #include <string.h>
@@ -67,8 +68,7 @@ int filespace_init(struct filespace *fs, int initial_size) {
   if (initial_size == 0)
     initial_size = 1024;
   fs->current_alloc = initial_size;
-  fs->str = (char *) malloc(fs->current_alloc);
-  if (!fs->str) return -1;
+  fs->str = (char *) safe_malloc(fs->current_alloc);
   fs->str[0] = '\0';
   fs->pos = fs->str;
   return 0;
@@ -102,9 +102,7 @@ if (len == 0) return 0;
   if (fs->current_alloc - fs->current_size < len + 2) {
     fs->current_alloc = (int) (fs->current_alloc * 1.4 + 1 );
     fs->current_alloc += 100 + len;
-    tmpstr = (char *) malloc(fs->current_alloc);
-    if (!tmpstr)
-      return -1;
+    tmpstr = (char *) safe_malloc(fs->current_alloc);
     memcpy(tmpstr, fs->str, fs->current_size);
     fs->pos = (fs->pos - fs->str) + tmpstr;
     if (fs->str) free(fs->str);
